import { Fn } from "./TypeUtils";
export declare function from_1_to_n(n: number): number[];
export declare function from_0_to_n(n: number): number[];
export declare function from_n_to_m(n: number, m: number): number[];
export declare function zipWith<A, B, C>(fn: (a: A, b: B) => C, arr1: A[], arr2: B[]): C[];
export declare function zip<A, B, C>(arr1: A[], arr2: B[]): [A, B][];
export declare function exclude<A>(arr: A[], exlusionCriteria: (elem: A) => boolean): A[];
export declare function mutating_delete_at<T>(array: T[], index: number): T[];
export declare function minimum_by<A>(array: A[], sortFn: (a: A, b: A) => number): A;
export declare function maximum_by<A>(array: A[], sortFn: (a: A, b: A) => number): A;
export declare function toArrayLiteralString(array: readonly number[] | readonly string[]): string;
/**
 * Given a
 * @param type_guard , apparently now called "type predicate", a (ElemT => boolean) function with special return type annotation
 *                   that can assert the type of an element in a way the compiler respects
 *                   (see <https://www.typescriptlang.org/docs/handbook/2/narrowing.html#using-type-predicates>)
 * this function (returns a function that) filters an array to just those that pass the type guard,
 * and makes TS believe the fact that the resulting array is of the asserted type.
 * NB: if the type guard is incomplete/incorrect, the resulting array may not actually contain only elements of the asserted type.
 * TODO: maybe adapt API from
 *  filter_and_assert_type(type_guard)(array) to
 *  filter_and_assert_type(array, type_guard)
 * Also, what would it take to allow
 *  array.filter_and_assert_type(type_guard) ?
 */
export declare function filter_and_assert_type<ElemT, AssertedT extends ElemT>(type_guard: ((el: ElemT) => el is AssertedT)): Fn<ElemT[], AssertedT[]>;
